use crate::client::Navidrome;
use std::error::Error;
use std::fmt::{Debug, Display, Formatter};

pub struct NavidromeBuilder {
    base: Option<String>,
    token: Option<String>,
}

#[derive(Debug)]
pub enum NavidromeBuilderError {
    MissingParam,
    Reqwest(reqwest::Error),
}

impl Display for NavidromeBuilderError {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        write!(f, "{:?}", self)
    }
}

impl Error for NavidromeBuilderError {}

impl NavidromeBuilder {
    pub fn new() -> NavidromeBuilder {
        return NavidromeBuilder {
            base: None,
            token: None,
        };
    }

    pub fn base(&mut self, base: impl ToString) -> &mut NavidromeBuilder {
        self.base = Some(base.to_string());
        return self;
    }

    pub fn token(&mut self, token: impl ToString) -> &mut NavidromeBuilder {
        self.token = Some(token.to_string());
        return self;
    }

    pub fn build(&self) -> Result<Navidrome, NavidromeBuilderError> {
        let client = reqwest::ClientBuilder::new()
            .build()
            .map_err(|e| NavidromeBuilderError::Reqwest(e))?;

        return Ok(Navidrome {
            base: self
                .base
                .clone()
                .ok_or(NavidromeBuilderError::MissingParam)?,
            http: client,
            token: self
                .token
                .clone()
                .ok_or(NavidromeBuilderError::MissingParam)?,
        });
    }
}
