use navidrome::{
    client::{library::Library, Navidrome},
    models::navidrome::NavidromeTrack,
};

use std::collections::HashMap;
use std::sync::Arc;
use std::time::Duration;
use tokio::sync::RwLock;
use tokio::time::{interval, MissedTickBehavior};

pub async fn update_tracks_daemon(
    media_client: &Navidrome,
    tracks_map: &mut Arc<RwLock<HashMap<String, NavidromeTrack>>>,
) -> Result<(), Box<dyn std::error::Error>> {
    println!("Tracks daemon starting...");

    let mut ticker = interval(Duration::from_secs(30));
    ticker.set_missed_tick_behavior(MissedTickBehavior::Skip);

    loop {
        let mut tracks_map = tracks_map.write().await;
        media_client.tracks().await?.iter().for_each(|t| {
            if let Some(mb_id) = &t.musicbrainz_track_id {
                tracks_map.insert(mb_id.clone(), t.clone());
            }
        });

        println!("Found {} tracks", tracks_map.len());

        ticker.tick().await;
    }
}
